import asyncio
import logging
import aiohttp.web

from . import endpoints

LOG = logging.getLogger(__name__)


class SirBot(aiohttp.web.Application):
    def __init__(self, user_agent=None, **kwargs):
        super().__init__(**kwargs)

        self.plugins = dict()
        self.http_session = aiohttp.ClientSession(loop=kwargs.get('loop') or asyncio.get_event_loop())
        self.user_agent = user_agent or 'sir-bot-a-lot'
        self.router.add_route('GET', '/sirbot/plugins', endpoints.plugins)

    def start(self, **kwargs):
        LOG.info('Starting SirBot')
        aiohttp.web.run_app(self, **kwargs)

    def load_plugin(self, plugin):
        self.plugins[plugin.__name__] = plugin
        plugin.load(self)

    async def shutdown(self):
        LOG.info('Stoppping SirBot')
        self.http_session.close()
        await super().shutdown()
        LOG.info('SirBot stopped')



import sys
import pathlib
import setuptools


if sys.version_info < (3, 6):
    raise RuntimeError('SirBot requires Python 3.6+')

LONG_DESCRIPTION = pathlib.Path('README.rst').read_text('utf-8')

requires = {
    'install': ['aiohttp'],
    'slack': ['slack-sansio'],
    'github': ['gidgethub'],
    'setup': ['pytest-runner'],
    'tests': ['flake8', 'pytest>=3.3.0', 'coverage', 'pytest-coverage', 'pytest-asyncio', 'asynctest'],
    'full': set(),
    'doc': {'sphinx', 'sphinxcontrib-asyncio', 'sphinxcontrib-napoleon'},
    'dev': {'tox'},
}
requires['dev'].update(*requires.values())
requires['full'].update(requires['install'], requires['slack'], requires['github'])


def find_version():
    with open("sirbot/__version__.py") as f:
        version = f.readlines()[-1].split('=')[-1].strip().strip("'").strip('"')
        if not version:
            raise RuntimeError('No version found')

    return version


setuptools.setup(
    name='sirbot',
    long_description=LONG_DESCRIPTION,
    description='Rewrite of sir-bot-a-lot',
    keywords=[
        'sirbot',
        'chatbot',
        'bot',
        'slack',
    ],
    packages=setuptools.find_packages(),
    zip_safe=True,
    python_requires='~=3.6',
    install_requires=requires['install'],
    setup_requires=requires['setup'],
    tests_require=requires['tests'],
    extras_require=requires,
    # See: http://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
    ],
    author=(
        'Ovv <contact@ovv.wtf>',
    ),
    author_email='contact@ovv.wtf',
    license='MIT',
    url='https://github.com/pyslackers/sir-bot-a-lot2',
    version=find_version(),
)



import os
import logging

from gidgethub import ValidationFailure
from gidgethub.aiohttp import GitHubAPI
from gidgethub.sansio import Event
from gidgethub.routing import Router

from aiohttp.web import Response

LOG = logging.getLogger(__name__)


class GithubPlugin:
    __name__ = 'github'

    def __init__(self, *, verify=None):
        self.api = None
        self.router = Router()
        self.verify = verify or os.environ['GITHUB_VERIFY']

    def load(self, sirbot):
        LOG.info('Loading github plugin')
        self.api = GitHubAPI(session=sirbot.http_session, requester=sirbot.user_agent)

        sirbot.router.add_route('POST', '/github', dispatch)


async def dispatch(request):
    github = request.app.plugins['github']
    payload = await request.read()

    try:
        event = Event.from_http(request.headers, payload, secret=github.verify)
        await github.router.dispatch(event, app=request.app)
    except ValidationFailure:
        LOG.debug('Github webhook failed verification: %s, %s', request.headers, payload)
    except Exception as e:
        LOG.exception(e)
        return Response(status=500)
    else:
        return Response(status=200)



import asyncio
import logging

from aiohttp.web import Response
from slack.events import Event
from slack.actions import Action
from slack.commands import Command
from slack.exceptions import FailedVerification

LOG = logging.getLogger(__name__)


async def incoming_event(request):
    slack = request.app.plugins['slack']
    payload = await request.json()
    LOG.log(5, 'Incoming event payload: %s', payload)

    if 'challenge' in payload:
        if payload['token'] == slack.verify:
            return Response(body=payload['challenge'])
        else:
            return Response(status=500)

    try:
        event = Event.from_http(payload, verification_token=slack.verify)
    except FailedVerification:
        return Response(status=401)

    if event['type'] == 'message':
        callbacks = list(_incoming_message(event, request))
    else:
        callbacks = list(_incoming_event(event, request))

    if callbacks:
        try:
            asyncio.wait(callbacks, return_when=asyncio.ALL_COMPLETED)
        except Exception as e:
            LOG.exception(e)
            return Response(status=500)

    return Response(status=200)


def _incoming_event(event, request):
    LOG.debug('Incoming event: %s', event)
    slack = request.app.plugins['slack']
    for handler in slack.routers['event'].dispatch(event):
        yield asyncio.ensure_future(handler(event, request.app))


def _incoming_message(event, request):
    slack = request.app.plugins['slack']
    if event.get('bot_id') == slack.bot_id or event.get('message', {}).get('bot_id') == slack.bot_id:
        return

    LOG.debug('Incoming message: %s', event)
    mention = slack.bot_user_id in event.get('text', '') or event['channel'].startswith('D')

    if mention and 'text' in event:
        event['text'] = event['text'].strip('<@{}>'.format(slack.bot_user_id)).strip()

    for handler in slack.routers['message'].dispatch(event):
        option = slack.handlers_option[handler]
        if option['mention'] and not mention:
            continue
        elif option['admin'] and event['user'] not in slack.admins:
            continue

        yield asyncio.ensure_future(handler(event, request.app))


async def incoming_command(request):
    slack = request.app.plugins['slack']
    payload = await request.post()

    try:
        command = Command(payload, verification_token=slack.verify)
    except FailedVerification:
        return Response(status=401)

    LOG.debug('Incoming command: %s', command)
    callbacks = list()
    for callback in slack.routers['command'].dispatch(command):
        callbacks.append(asyncio.ensure_future(callback(command, request.app)))

    if callbacks:
        try:
            asyncio.wait(callbacks, return_when=asyncio.ALL_COMPLETED)
        except Exception as e:
            LOG.exception(e)
            return Response(status=500)

    return Response(status=200)


async def incoming_actions(request):
    slack = request.app.plugins['slack']
    payload = await request.post()
    LOG.log(5, 'Incoming action payload: %s', payload)

    try:
        action = Action.from_http(payload, verification_token=slack.verify)
    except FailedVerification:
        return Response(status=401)

    LOG.debug('Incoming action: %s', action)
    callbacks = list()
    for callback in slack.routers['action'].dispatch(action):
        callbacks.append(asyncio.ensure_future(callback(action, request.app)))

    if callbacks:
        try:
            asyncio.wait(callbacks, return_when=asyncio.ALL_COMPLETED)
        except Exception as e:
            LOG.exception(e)
            return Response(status=500)

    return Response(status=200)



import os
import logging
import asyncio

from slack import methods
from slack.io.aiohttp import SlackAPI
from slack.actions import Router as ActionRouter
from slack.commands import Router as CommandRouter
from slack.events import EventRouter, MessageRouter

from . import endpoints

LOG = logging.getLogger(__name__)


class SlackPlugin:
    __name__ = 'slack'

    def __init__(self, *, token=None, verify=None, bot_id=None, bot_user_id=None, admins=None):
        self.api = None
        self.token = token or os.environ['SLACK_TOKEN']
        self.admins = admins or os.environ.get('SLACK_ADMINS', [])
        self.verify = verify or os.environ['SLACK_VERIFY']
        self.bot_id = bot_id or os.environ.get('SLACK_BOT_ID')
        self.bot_user_id = bot_user_id or os.environ.get('SLACK_BOT_USER_ID')
        self.handlers_option = {}

        if not self.bot_user_id and not self.bot_id:
            LOG.warning('`SLACK_BOT_USER_ID` or `SLACK_BOT_ID` required to use `on_mention` routing.')

        self.routers = {
            'event': EventRouter(),
            'command': CommandRouter(),
            'message': MessageRouter(),
            'action': ActionRouter(),
        }

    def load(self, sirbot):
        LOG.info('Loading slack plugin')
        self.api = SlackAPI(session=sirbot.http_session, token=self.token)

        sirbot.router.add_route('POST', '/slack/events', endpoints.incoming_event)
        sirbot.router.add_route('POST', '/slack/commands', endpoints.incoming_command)
        sirbot.router.add_route('POST', '/slack/actions', endpoints.incoming_actions)

        if self.bot_user_id and not self.bot_id:
            sirbot.on_startup.append(self.find_bot_id)
        elif not self.bot_user_id and not self.bot_id:
            sirbot.on_startup.append(self.find_bot_user_id)

    def on_event(self, event_type, handler):
        if not asyncio.iscoroutinefunction(handler):
            handler = asyncio.coroutine(handler)
        self.routers['event'].register(event_type, handler)

    def on_command(self, command, handler):
        if not asyncio.iscoroutinefunction(handler):
            handler = asyncio.coroutine(handler)
        self.routers['command'].register(command, handler)

    def on_message(self, pattern, handler, flags=0, channel='*', mention=False, admin=False):
        if not asyncio.iscoroutinefunction(handler):
            handler = asyncio.coroutine(handler)

        if admin and not self.admins:
            LOG.warning('Slack admins ids are not set. Admin limited endpoint will not work.')

        self.handlers_option[handler] = {'mention': mention, 'admin': admin}
        self.routers['message'].register(pattern=pattern, handler=handler, flags=flags, channel=channel)

    def on_action(self, action, handler, name='*'):
        if not asyncio.iscoroutinefunction(handler):
            handler = asyncio.coroutine(handler)
        self.routers['action'].register(action, handler, name)

    async def find_bot_id(self, app):
        rep = await self.api.query(url=methods.USERS_INFO, data={'user': self.bot_user_id})
        self.bot_id = rep['user']['profile']['bot_id']

    async def find_bot_user_id(self, app):
        rep = await self.api.query(
            url=methods.CHAT_POST_MESSAGE,
            data={'channel': 'general', 'text': 'Looking for bot id'}
        )
        LOG.warning('The BOT_USER_ID is : "%s"', rep['message']['user'])
